function w = ir(SHOCK, volshock, T, stocvol, sigz, sigv, ubar, LAMu, V,...
                gx, hx, hxi, gxx, hxx, guu, huu, gzu, hzu, gzz, hzz, gvv, hvv)
% Purpose:  This code computes T-period impulse-response functions for the vector 
%                   w=[y x] 
%           of variables, solving a DSGE model with time-varying risk up to a second-order 
%           approximation. For details see: "Second-Order Approximation of Dynamic Models with 
%           Time-Varying Risk", by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic (2010)
%
%           The model is 
%                   E_t{f(yp,xp,y,x)}=0
% 
%           The law of motion for vector w is:
%          
%   y  = gx*x + .5*[(Iyx')*gxx*x + (Iyu')*guu*u + gvv*sigv^2 + gzz*sigz^2 + 2*gzu*sigz*u]
%   xp = hx*x + .5*[(Ixx')*hxx*x + (Ixu')*huu*u + hvv*sigv^2 + hzz*sigz^2 + 2*hzu*sigz*u] + hxi*xip
% 
%           for the case of stochastic STANDARD DEVIATIONS, and
% 
%   y  = gx*x + .5*[(Iyx')*gxx*x + guu*u^2 + gzz*sigz^2]
%   xp = hx*x + .5*[(Ixx')*hxx*x + huu*u^2 + hzz*sigz^2] + hxi*xip
%
%           for that of stochastic VARIANCES
% 
% Input:    SHOCK:  element of z to be shocked
%           volshock: scalar selecting class of shocks (0=structural shock; 1=volatility shock)
%           T:      length of irfs
%           stocvol: model for stochastic volatility
%           sigz:   scale parameter for structural shocks
%           sigv:   scale parameter for volatility shocks
%           ubar:   vector scaling the several s.s. STANDARD DEVIATIONS wrt scale parameter sigma_z
%           LAMu:   persistence of volatility shocks
%           V:      scale matrix in the process of the volatility
%           hx:     the numerical value of the Jacobian matrix of h wrt x     
%           gx:     the numerical value of the Jacobian matrix of g wrt x      
%           hxi:    the matrix mapping the driving force xi into vector x  
%           hxx:    the numerical value of the Hessian matrix of h wrt x      
%           gxx:    the numerical value of the Hessian matrix of g wrt x      
%           hzz:    the numerical value of the Hessian matrix of h wrt z      
%           gzz:    the numerical value of the Hessian matrix of g wrt z      
%           hzu:    the numerical value of the Hessian matrix of h wrt zu     
%           gzu:    the numerical value of the Hessian matrix of g wrt zu     
%           hvv:    the numerical value of the Hessian matrix of h wrt v      
%           gvv:    the numerical value of the Hessian matrix of g wrt v      
%
% Output:   the impulse-response functions, with length T, as a matrix n+nz by T.
%
% Modifies and adapts a code written by Mike Woodford
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010, Revised March 2011
% 

% ****************************************************************
% [1] Compute the true impulse responses
% ****************************************************************

% Number of shocks
nz = size(hxi, 2);

% Columns of xt, yt and ut
nx = size(hx,1);
ny = size(gx,1);
nu = nz;

n = ny + nx;

% set xt, yt, and ut
y = zeros(ny, T);
x = zeros(nx, T);
u = zeros(nz, T);

Iuy = eye(ny);
Iux = eye(nx);

ez = zeros(nz,1);
ev = zeros(nz,1);

if volshock==0
    ez(SHOCK) = 1;
elseif volshock==1
    ev(SHOCK) = 1;
else
    ez(SHOCK) = 1;
    ev(SHOCK) = 1;
end

% % set initial conditions
utnot = zeros(nz,1);
xtnot = zeros(nx,1);

U = diag(sigz * ubar);
xi = U * ez;

%Decompose x into its first- and second-order components
xfnot = xtnot;
xsnot = 0*xtnot;

xf = zeros(nx, T);
xs = zeros(nx, T);

if stocvol==0
    
    xf(:, 1) = hx * xfnot + hxi * xi;
    xs(:, 1) = hx * xsnot + .5 * kron(eye(nx),xfnot') * hxx * xfnot;
    
    x(:, 1) = xf(:, 1) + xs(:, 1);

    y(:, 1) = gx * x(:, 1) + .5 * kron(eye(ny),xf(:, 1)') * gxx * xf(:, 1);
        
    for J = 2:T
        
    	xf(:, J) = hx * xf(:, J-1);
    	xs(:, J) = hx * xs(:, J-1) + .5 * kron(eye(nx),xf(:, J-1)') * hxx * xf(:, J-1);
        
        x(:, J) = xf(:, J) + xs(:, J);
        
        y(:, J) = gx * x(:, J) + .5 * kron(eye(ny),xf(:, J)') * gxx * xf(:, J);

    end
    
elseif stocvol==1

    xf(:, 1) = hx * xfnot + hxi * xi;
    xs(:, 1) = hx * xsnot + .5 * (kron(eye(nx),xfnot') * hxx * xfnot + kron(Iux,utnot') * huu * utnot...
            + 2 * hzu * sigz * utnot);

    x(:, 1) = xf(:, 1) + xs(:, 1);
    
    u(:, 1) = LAMu * utnot + sigv * V * ev;

    y(:, 1) = gx * x(:, 1) + .5 * (kron(eye(ny),xf(:, 1)') * gxx * xf(:, 1) + kron(Iuy,u(:, 1)') * guu * u(:, 1)...
            + 2 * gzu * sigz * u(:, 1));
        
    for J = 2:T
        
    	xf(:, J) = hx * xf(:, J-1);
    	xs(:, J) = hx * xs(:, J-1) + .5 * (kron(eye(nx),xf(:, J-1)') * hxx * xf(:, J-1) ...
            + kron(Iux,u(:,J-1)') * huu * u(:,J-1) + 2 * hzu * sigz * u(:,J-1));
        
        x(:, J) = xf(:, J) + xs(:, J);
        
        u(:, J) = LAMu * u(:, J-1);

     	y(:, J) = gx * x(:, J) + .5 * (kron(eye(ny),xf(:, J)') * gxx * xf(:, J) ...
            + kron(Iuy,u(:, J)') * guu * u(:, J) + 2 * gzu * sigz * u(:, J));

    end
    
elseif stocvol==2
    
    xf(:, 1) = hx * xfnot + hxi * xi;
    xs(:, 1) = hx * xsnot + .5 * (kron(eye(nx),xfnot') * hxx * xfnot + huu * utnot);

    x(:, 1) = xf(:, 1) + xs(:, 1);

    u(:, 1) = LAMu * utnot + sigv^2 * V * ev;

    y(:, 1) = gx * x(:, 1) + .5 * (kron(eye(ny),xf(:, 1)') * gxx * xf(:, 1) + guu * u(:, 1));
        
    for J = 2:T
        
    	xf(:, J) = hx * xf(:, J-1);
    	xs(:, J) = hx * xs(:, J-1) + .5 * (kron(eye(nx),xf(:, J-1)') * hxx * xf(:, J-1) + huu * u(:,J-1));
        
        x(:, J) = xf(:, J) + xs(:, J);
        
        u(:, J) = LAMu * u(:, J-1);

     	y(:, J) = gx * x(:, J) + .5 * (kron(eye(ny),xf(:, J)') * gxx * xf(:, J) + guu * u(:, J));

    end
        
end

w = [y; x; u];

% ****************************************************************
% [2] Check the magnitude of imaginary components of responses
% and suppress these components to issue report
% ****************************************************************

IMW=imag(w);

if max(max(abs(IMW))) > .0000001
        disp('Maximum imaginary component is larger than')
        disp('10 to the -8.  The magnitude of this part is:')
        disp(max(max(abs(IMW))))
        pause
end

w=real(w);

