function [yt, xt, ut] = simtimser(hxi, sigz, sigv, ubar, x0, ez, ev, LAMu, V, stocvol, approx, ...
                        gx, hx, gxx, hxx, guu, huu, gzu, hzu, gzz, hzz, gvv, hvv)
% Purpose:  Simulates time series from a DSGE model with time-varying volatility.
%           For details see: "Second-Order Approximation of Dynamic Models with Time-Varying Risk", 
%           by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic (2010)
% 
%           The model can be written as: 
%                   E_t{f(yp,xp,y,x)}=0, 
% 
%           The solution is of the form
% 
%                   y  = g(x,u,sigz,sigv)
%                   xp = h(x,u,sigz,sigv) + hxi*xip
% 
%           Stochastic volatility is modeled as:
% 
%                   xip = U*ezp
% 
%           where U=diag(u) is time-varying and either
% 
%                   up = sigz*(Iz-LAMu*ubar) + LAMu*u + sigv*V*evp
% 
%           for the case of stochastic STANDARD DEVIATIONS, or
% 
%                   up^2 = sigz^2*(Iz-LAMu*ubar^2) + LAMu*u^2 + sigv^2*V*evp
% 
%           for that of stochastic VARIANCES.
% 
%           The quadratic approximation to these functions are: 
% 
%                   y  = gx*x + .5*[(Iyx')*gxx*x + (Iyu')*guu*u + gvv*sigv^2 + gzz*sigz^2 + 2*gzu*sigz*u]
%                   xp = hx*x + .5*[(Ixx')*hxx*x + (Ixu')*huu*u + hvv*sigv^2 + hzz*sigz^2 + 2*hzu*sigz*u] + hxi*xip
% 
%           for the case of stochastic STANDARD DEVIATIONS, and
% 
%                   y  = gx*x + .5*[(Iyx')*gxx*x + guu*u^2 + gzz*sigz^2]
%                   xp = hx*x + .5*[(Ixx')*hxx*x + huu*u^2 + hzz*sigz^2] + hxi*xip
%
%           for that of stochastic VARIANCES
%          
%           [Notation:  y and x are in deviation from ss, * is matrix product,  is kronecker product, 
%                       Iy is the identity matrix of order ny, Ix the identity matrix of order nx]
% 
% Input:    solution of the approximation, where 
%                   gx is ny by nx
%                   hx is nx by nx
%                   gxx is ny*nx by nx
%                   hxx is nx*nx by nx
%                   guu is ny*nz by nz
%                   huu is nx*nz by nz
%                   gvv, gzz are ny by 1
%                   hvv, hzz are nx by 1
%                   gzu are ny by nz
%                   hzu are nx by nz
%                   hxi is nx by nz
%                   ubar is nz by 1
%                   LAMu is nz by nz
%                   V is nz by nz
%                   sigz, sigv are positive scalars, scaling the innovations to the
%                                   exogenous variables and the volatility processes
%                   ez, ev are T by nez random shocks
%                   T is the length of the simulation 
%
%                   x0 is the initial condition for xt
%
% Output:  matrices yt (T by ny), xt (T by nx) and ut (T by nu) containing time series for y, x and u
%
% (c) Gianluca Benigno, Pierpaolo Benigno, Salvatore Nistic
% March, 2011
% 


% Number of periods
T = size(ez,1)+1;

% Number of shocks
nez = size(hxi,2);

% Columns of xt, yt and ut
nx = size(hx,1);
ny = size(gx,1);
nu = nez;

% set xt, yt, and ut
yt = zeros(T,ny);
xt = zeros(T,nx);
ut = zeros(T,nu);

% set initial conditions
x0 = x0(:);

u0 = sigz * ubar;
u0 = u0(:);

xt(1,1:nx) = x0';
ut(1,1:nu) = u0';

%Decompose x into its first- and second-order components
xf = x0;
xs = 0*x0;

uf = u0;

if stocvol == 0
    
	gss = guu*ubar.^2 + gzz;
    hss = huu*ubar.^2 + hzz;

    for t=1:T

        %Construct y(t)
        if approx == 1
            y = gx * xf;
        elseif approx == 2
            y = gx * (xf+xs) + .5 * (kron(eye(ny),xf')) * gxx * xf + .5 * gss * sigz^2;
        end
    
        yt(t,1:ny) = y';

        %Remember xf(t) and xs(t) when entering t+1
        xoldf = xf; 
        xolds = xs;

        if t<T
        %Construct x(t+1) given x(t) and xi(t+1)
            xf = hx * xoldf + sigz * hxi * ez(t,:)';
            if approx == 2
                xs = hx * xolds + 1/2 * (kron(eye(nx),xoldf')) * hxx * xoldf + 1/2 * hss * sigz^2;
            end
                
            if approx == 1
                xt(t+1,:) = xf';
            elseif approx == 2
                xt(t+1,:) = (xf + xs)';
            end
        end
    end
    
elseif stocvol == 1
    
    for t=1:T

        %Construct y(t)
            if approx == 1
                y = gx * xf;
            elseif approx == 2
                y = gx * (xf+xs) + .5 * (kron(eye(ny),xf')) * gxx * xf ...
                    + .5 * (kron(eye(ny),uf')) * guu * uf ...
                    + .5 * gzz * sigz^2 + .5 * gvv * sigv^2 + gzu * sigz * uf;
            end
    
        yt(t,1:ny) = y';

        %Remember uf(t), xf(t) and xs(t) when entering t+1
        uoldf = uf; 
        xoldf = xf; 
        xolds = xs;

        if t<T
        %Construct xi(t+1) given u(t)
            xip = diag(uf) * ez(t,:)';
        
        %Construct x(t+1) given x(t), u(t) and xi(t+1)
            xf = hx * xoldf + hxi * xip;
            if approx == 2
                xs = hx * xolds + .5 * (kron(eye(nx),xoldf')) * hxx * xoldf ...
                    + .5 * (kron(eye(ny),uoldf')) * huu * uoldf ...
                    + .5 * hzz * sigz^2 + .5 * hvv * sigv^2 + hzu * sigz * uoldf;
            end
                
            if approx == 1
                xt(t+1,:) = xf';
            elseif approx == 2
                xt(t+1,:) = (xf + xs)';
            end
            
        %Construct u(t+1) given u(t)
            uf = sigz * (eye(nu) - LAMu) * ubar + LAMu * uoldf + sigv * V * ev(t,:)';
            
            ut(t+1,:) = uf';
        end
    end
    
elseif stocvol == 2
    
    u0 = sigz^2 * ubar.^2;
    u0 = u0(:);
    uf = u0;

    ut(1,1:nu) = uf';
    
    for t=1:T

        %Construct y(t)
        if approx == 1
            y = gx * xf;
        elseif approx == 2
            y = gx * (xf+xs) + .5 * (kron(eye(ny),xf')) * gxx * xf ...
                + .5 * guu * uf + .5 * gzz * sigz^2;
        end
    
        yt(t,1:ny) = y';

        %Remember uf(t), xf(t) and xs(t) when entering t+1
        uoldf = uf; 
        xoldf = xf; 
        xolds = xs;

        if t<T
        %Construct xi(t+1) given u(t)
            xip = diag(sqrt(uf)) * ez(t,:)';
        
        %Construct x(t+1) given x(t), u(t) and xi(t+1)
            xf = hx * xoldf + hxi * xip;
            if approx == 2
                xs = hx * xolds + .5 * (kron(eye(nx),xoldf')) * hxx * xoldf ...
                    + .5 * huu * uoldf + .5 * hzz * sigz^2;
            end
                
            if approx == 1
                xt(t+1,:) = xf';
            elseif approx == 2
                xt(t+1,:) = (xf + xs)';
            end
            
        %Construct u(t+1) given u(t)
            uf = sigz^2 * (eye(nu) - LAMu) * ubar.^2 + LAMu * uoldf + sigv^2 * V * ev(t,:)';
            
            ut(t+1,:) = uf';
        end
    end
end
