function [gxx,hxx,guu,huu,gzu,hzu,gzz,hzz,gvv,hvv] = soappr_std(Df,Hf,hx,gx,hxi,LAMu,ubar,VV)
% Purpose:  Computes the second-order accurate approximation of the solution to a DSGE model 
%           with time-varying risk: linear process for the conditional STANDARD DEVIATION
%           For details see: "Second-Order Approximation of Dynamic Models with Time-Varying Risk", 
%           by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic (2010)
%
%           The model is 
%                   E_t{f(yp,xp,y,x)}=0
% 
%           Stochastic volatility is modeled by means of a linear process
%           for the conditional STANDARD DEVIATION of structural shocks:
% 
%                   up = sigz*(Iz-LAMu*ubar) + LAMu*u + sigv*V*evp
%          
% Input:    Df:  the numerical value of the Jacobian matrix of function f  (n by 2n)
%           Hf:  the numerical value of the Magnus-Neudecker Hessian of f  (2*n^2 by 2*n)
%           hx:  the numerical value of the Jacobian matrix of h wrt x     (nx by nx)
%           gx:  the numerical value of the Jacobian matrix of g wrt x     (ny by nx) 
%           hxi: selects the driving force from x   (nx by nz) 
%
% For the definition of the Jacobian matrix and the Hessian, see:
%           Magnus and Neudecker: "Matrix Differential Calculus with 
%           Applications in Statistics and Econometrics".
%
% Output:   the matrices gxx, hxx, gzz, hzz, guu, huu, gvv, hvv, gzu, hzu
%           which characterize the second-order approximation of the policy
%           functions, as follows:
%
%   y  = gx*x + .5*[(Iyx')*gxx*x + (Iyu')*guu*u + gvv*sigv^2 + gzz*sigz^2 + 2*gzu*sigz*u]
%   xp = hx*x + .5*[(Ixx')*hxx*x + (Ixu')*huu*u + hvv*sigv^2 + hzz*sigz^2 + 2*hzu*sigz*u] + hxi*xip
%
% [Notation:    y and x are in deviation from ss, * is matrix product,  is kronecker product, 
%               Iy is the identity matrix of order ny, Ix the identity matrix of order nx]
% 
% Calls:    tracem.m
%           diagmn.m
%
% Modifies and adapts a code written by Paul Gomme and Paul Klein
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010 
% 

% set up the tools
% -------------------------------------------------------------------------
n  = size(Df,1);        % Number of variables
nx = size(hx,1);        % Number of state variables
nz = size(hxi,2);       % Number of exogenous state variables
ny = n-nx;			 	% Number of jump variables

fyp = Df(:,1:ny);       % numerical gradient wrt yp
fxp = Df(:,ny+1:n);     % numerical gradient wrt xp
fy  = Df(:,n+1:n+ny);   % numerical gradient wrt y

Hfyp = Hf(1:n*ny,:);         % numerical MN Hessian wrt yp
Hfxp = Hf(n*ny+1:n*n,:);     % numerical MN Hessian wrt xp
Hfy  = Hf(n*n+1:n*n+n*ny,:); % numerical MN Hessian wrt y
Hfx  = Hf(n*n+n*ny+1:end,:); % numerical MN Hessian wrt x

% set matrices Mx and Mxi
mx  = [gx*hx; hx; gx; eye(nx)];
mxi = [gx*hxi; hxi; zeros(n,nz)];

% compute the numerical value for gxx and hxx
% -------------------------------------------------------------------------
aax = [kron(eye(n),hx'*gx'), kron(eye(n),hx'), kron(eye(n),gx'), kron(eye(n),eye(nx))]*Hf*mx;
% aax = kron(eye(n),hx'*gx')*Hfyp*mx + kron(eye(n),hx')*Hfxp*mx ...
%         + kron(eye(n),gx')*Hfy*mx + kron(eye(n),eye(nx))*Hfx*mx;

bbx = kron((fyp*gx+fxp),eye(nx));
ccx = kron(fyp,hx');
ddx = kron(fy,eye(nx));

eex = [kron(eye(nx),ddx)+kron(hx',ccx) kron(eye(nx),bbx)];
solx = -eex\aax(:);

gxx = reshape(solx(1:nx^2*ny),nx*ny,nx);
hxx = reshape(solx(nx^2*ny+1:end),nx^2,nx);

% compute the numerical value for guu and huu
% -------------------------------------------------------------------------
aau = diagmn((kron(fyp,hxi'))*gxx*hxi + (kron(eye(n),hxi'*gx'))*Hfyp*mxi + (kron(eye(n),hxi'))*Hfxp*mxi);

bbu = kron((fyp*gx+fxp),eye(nz));
ccu = kron(fyp,LAMu');
ddu = kron(fy,eye(nz));

eeu = [kron(eye(nz),ddu)+kron(LAMu',ccu) kron(eye(nz),bbu)];
solu = -eeu\aau(:);

guu = reshape(solu(1:nz^2*ny),nz*ny,nz);
huu = reshape(solu(nz^2*ny+1:end),nz*nx,nz);

% compute the numerical value for gzu and hzu
% -------------------------------------------------------------------------
aazu = (kron(fyp,ubar'*(eye(nz)-LAMu)'))*guu*LAMu;

bbzu = fyp*gx+fxp;
cczu = fyp;
ddzu = fy;

eezu = [kron(eye(nz),ddzu)+kron(LAMu',cczu) kron(eye(nz),bbzu)];
solzu = -eezu\aazu(:);

gzu = reshape(solzu(1:nz*ny),ny,nz);
hzu = reshape(solzu(nz*ny+1:end),nx,nz);

% compute the numerical value for gzz and hzz
% -------------------------------------------------------------------------
aaz = (kron(fyp,ubar'*(eye(nz)-LAMu)'))*guu*(eye(nz)-LAMu)*ubar + 2*fyp*gzu*(eye(nz)-LAMu)*ubar;

bbz = fyp*gx+fxp;
ccz = fyp + fy;

eez = [ccz bbz];
solz = -eez\aaz;

gzz = solz(1:ny);
hzz = solz(ny+1:end);

% compute the numerical value for gvv and hvv
% -------------------------------------------------------------------------
aav = tracem((kron(fyp,VV'))*guu*VV);

bbv = fyp*gx+fxp;
ccv = fyp + fy;

eev = [ccv bbv];
solv = -eev\aav;

gvv = solv(1:ny);
hvv = solv(ny+1:end);

