function [gxx,hxx,guu,huu,gzz,hzz] = soappr_var(Df,Hf,hx,gx,hxi,LAMu,ubar,VV)
% Purpose:  Computes the second-order accurate approximation of the solution to a DSGE model 
%           with time-varying risk: linear process for the conditional VARIANCE
%           For details see: "Second-Order Approximation of Dynamic Models with Time-Varying Risk", 
%           by Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic (2010)
%
%           The model is 
%                   E_t{f(yp,xp,y,x)}=0
%          
%           Stochastic volatility is modeled by means of a linear process
%           for the conditional VARIANCE of structural shocks:
% 
%                   up^2 = sigz^2*(Iz-LAMu)*ubar^2 + LAMu*u^2 + sigv*V*evp
%          
% Input:    Df:  the numerical value of the Jacobian matrix of function f  (n by 2n)
%           Hf:  the numerical value of the Magnus-Neudecker Hessian of f  (2*n^2 by 2*n)
%           hx:  the numerical value of the Jacobian matrix of h wrt x     (nx by nx)
%           gx:  the numerical value of the Jacobian matrix of g wrt x     (ny by nx) 
%           hxi: selects the driving force from x   (nx by nz) 
%
% For the definition of the Jacobian matrix and the Hessian, see:
%           Magnus and Neudecker: "Matrix Differential Calculus with 
%           Applications in Statistics and Econometrics".
%
% Output:   the matrices gxx, hxx, gzz, hzz, guu, huu
%           which characterize the second-order approximation of the policy
%           functions, as follows:
%
%   y  = gx*x + .5*[(Iyx')*gxx*x + guu*u^2 + gzz*sigz^2]
%   xp = hx*x + .5*[(Ixx')*hxx*x + huu*u^2 + hzz*sigz^2] + hxi*xip
%
% [Notation:    y and x are in deviation from ss, * is matrix product,  is kronecker product, 
%               Iy is the identity matrix of order ny, Ix the identity matrix of order nx]
% 
% Calls:    diagm.m
%
% Modifies and adapts a code written by Paul Gomme and Paul Klein
% 
% (c) Gianluca Benigno, Pierpaolo Benigno and Salvatore Nistic
% November 2010, Revised March 2011
% 

% set up the tools
% -------------------------------------------------------------------------
n  = size(Df,1);        % Number of variables
nx = size(hx,1);        % Number of state variables
nz = size(hxi,2);       % Number of exogenous state variables
ny = n-nx;			 	% Number of jump variables

fyp = Df(:,1:ny);       % numerical gradient wrt yp
fxp = Df(:,ny+1:n);     % numerical gradient wrt xp
fy  = Df(:,n+1:n+ny);   % numerical gradient wrt y

Hfyp = Hf(1:n*ny,:);         % numerical MN Hessian wrt yp
Hfxp = Hf(n*ny+1:n*n,:);     % numerical MN Hessian wrt xp
Hfy  = Hf(n*n+1:n*n+n*ny,:); % numerical MN Hessian wrt y
Hfx  = Hf(n*n+n*ny+1:end,:); % numerical MN Hessian wrt x

% set matrices Mx and Mxi
mx  = [gx*hx; hx; gx; eye(nx)];
mxi = [gx*hxi; hxi; zeros(n,nz)];

% compute the numerical value for gxx and hxx
% -------------------------------------------------------------------------
aax = kron(eye(n),hx'*gx')*Hfyp*mx + kron(eye(n),hx')*Hfxp*mx ...
        + kron(eye(n),gx')*Hfy*mx + kron(eye(n),eye(nx))*Hfx*mx;

bbx = kron((fyp*gx),eye(nx)) + kron(fxp,eye(nx));
ccx = kron(fyp,hx');
ddx = kron(fy,eye(nx));

eex = [kron(eye(nx),ddx)+kron(hx',ccx) kron(eye(nx),bbx)];
solx = -eex\aax(:);

gxx = reshape(solx(1:nx^2*ny),nx*ny,nx);
hxx = reshape(solx(nx^2*ny+1:end),nx^2,nx);

% Using LAPACK's generalized Sylvester equation solver
% to find gxx and hxx

% sylaa = [ddx bbx];
% sylbb = - hx;
% sylcc = - aax;
% syldd = [ccx zeros(nx*(nx+ny),nx^2)];
% sylee = eye(nx);
% sylff = zeros(nx*(nx+ny),nx);
% 
% [sylrr,sylll] = gensyl(sylaa,sylbb,sylcc,syldd,sylee,sylff);
% gxx = sylrr(1:nx*ny,:);
% hxx = sylrr(nx*ny+1:end,:);

% compute the numerical value for guu and huu
% -------------------------------------------------------------------------
aau = (diagm((kron(fyp,hxi'))*gxx*hxi + (kron(eye(n),hxi'*gx'))*Hfyp*mxi + (kron(eye(n),hxi'))*Hfxp*mxi))';

bbu = fyp*gx+fxp;
ccu = fyp;
ddu = fy;

eeu = [kron(eye(nz),ddu)+kron(LAMu',ccu) kron(eye(nz),bbu)];
solu = -eeu\aau(:);

guu = reshape(solu(1:nz*ny),ny,nz);
huu = reshape(solu(nz*ny+1:end),nx,nz);

% compute the numerical value for gzz and hzz
% -------------------------------------------------------------------------
aaz = fyp*guu*(eye(nz)-LAMu)*ubar.^2;

bbz = fyp*gx+fxp;
ccz = fyp + fy;

eez = [ccz bbz];
solz = -eez\aaz;

gzz = solz(1:ny);
hzz = solz(ny+1:end);

